@@ -1,9 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: canon.scm,v 1.25 2003/02/14 18:28:01 cph Exp $
-
-Copyright 1988,1989,1990,1991,1992,1993 Massachusetts Institute of Technology
-Copyright 2001,2002,2003 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -19,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -90,11 +90,13 @@ ARBITRARY:	The expression may be executed more than once.  It
   splice?)				; top level can be moved
 
 (define *top-level-declarations*)
+(define *top-level-definitions*)
 
 (define (canonicalize/top-level expression)
   (if (eq? compiler:package-optimization-level 'NONE)
-      expression
-      (fluid-let ((*top-level-declarations* '()))
+      (values expression #f)
+      (fluid-let ((*top-level-declarations* '())
+		  (*top-level-definitions* '()))
 	(let ((result
 	       (canonicalize/expression
 		expression '()
@@ -102,11 +104,13 @@ ARBITRARY:	The expression may be executed more than once.  It
 			 (not (eq? compiler:package-optimization-level 'LOW)))
 		    'TOP-LEVEL
 		    'FIRST-CLASS))))
-	  (if (canout-needs? result)
-	      (canonicalize/bind-environment (canout-expr result)
-					     (scode/make-the-environment)
-					     expression)
-	      (canout-expr result))))))
+	  (values
+	   (if (canout-needs? result)
+	       (canonicalize/bind-environment (canout-expr result)
+					      (scode/make-the-environment)
+					      expression)
+	       (canout-expr result))
+	   *top-level-definitions*)))))
 
 (define (canonicalize/optimization-low? context)
   (or (eq? context 'FIRST-CLASS)
@@ -301,7 +305,7 @@ ARBITRARY:	The expression may be executed more than once.  It
 	(if (memq context '(ONCE-ONLY ARBITRARY))
 	    (error "canonicalize/definition: unscanned definition"
 		   expression))
-	(single-definition name value)))))
+	(single-definition name value context)))))
 
 (define (canonicalize/the-environment expr bound context)
   expr bound context ;; ignored
@@ -369,7 +373,10 @@ ARBITRARY:	The expression may be executed more than once.  It
 	 name
 	 (canout-expr value))))
 
-(define (single-definition name value)
+(define (single-definition name value context)
+  (if (and (eq? context 'TOP-LEVEL)
+	   (not (memq name *top-level-definitions*)))
+      (set! *top-level-definitions* (cons name *top-level-definitions*)))
   (make-canout (%single-definition name value)
 	       (canout-safe? value)
 	       true
@@ -378,7 +385,7 @@ ARBITRARY:	The expression may be executed more than once.  It
 ;; To reduce code space, split into two blocks, one with constants,
 ;; the other with expressions to be evaluated.
 
-(define (multi-definition names* values*)
+(define (multi-definition names* values* context)
   (define (collect names values wrapper)
     (if (null? (cdr values))
 	(%single-definition (car names) (car values))
@@ -404,6 +411,9 @@ ARBITRARY:	The expression may be executed more than once.  It
 	       (scode/comment-directive? (scode/comment-text value)
 					 'COMPILE-PROCEDURE)))))
 
+  (if (eq? context 'TOP-LEVEL)
+      (set! *top-level-definitions*
+	    (lset-union eq? names* *top-level-definitions*)))
   (let loop ((names names*) (values values*) (last 'NONE)
 	     (knames '()) (kvals '()) (vnames '()) (vvals '()))
     (cond ((null? names)
@@ -459,12 +469,14 @@ ARBITRARY:	The expression may be executed more than once.  It
 		 ((null? (cdr group))
 		  (let ((element (car group)))
 		    (cons (single-definition (car element)
-					     (cadr element))
+					     (cadr element)
+					     context)
 			  groups)))
 		 (else
 		  (let ((group (reverse group)))
 		    (cons (multi-definition (map car group)
-					    (map cadr group))
+					    (map cadr group)
+					    context)
 			  groups)))))
 
 	 (define (collect actions groups group)
@@ -499,7 +511,8 @@ ARBITRARY:	The expression may be executed more than once.  It
 						group)))
 				(else
 				 (collect (cdr actions)
-					  (cons (single-definition name value*)
+					  (cons (single-definition name value*
+								   context)
 						(add-group group groups))
 					  '()))))))))))
 
