@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: decls.scm,v 1.12 2007/01/05 21:19:20 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -198,11 +197,10 @@ USA.
 	      (and binary (< source binary) binary))))
      (set-source-node/modification-time! node modification-time)
      (if (not modification-time)
-	 (begin
-	   (fresh-line)
-	   (write-string "Source file newer than binary: ")
-	   (write (source-node/filename node))
-	   (newline)))))
+	 (write-notification-line
+	  (lambda (port)
+	    (write-string "Source file newer than binary: " port)
+	    (write (source-node/filename node) port))))))
    source-nodes)
   (if compiler:enable-integration-declarations?
       (begin
@@ -218,13 +216,14 @@ USA.
 				   (or (not time*)
 				       (> time* time)))))
 			    (if newer?
-				(begin
-				  (fresh-line)
-				  (write-string "Binary file ")
-				  (write (source-node/filename node))
-				  (write-string " newer than dependency ")
-				  (write (source-node/filename node*))
-				  (newline)))
+				(write-notification-line
+				 (lambda (port)
+				   (write-string "Binary file " port)
+				   (write (source-node/filename node) port)
+				   (write-string " newer than dependency "
+						 port)
+				   (write (source-node/filename node*)
+					  port))))
 			    newer?))))
 		 (set-source-node/modification-time! node #f))))
 	 source-nodes)
@@ -233,13 +232,12 @@ USA.
 	   (if (not (source-node/modification-time node))
 	       (for-each (lambda (node*)
 			   (if (source-node/modification-time node*)
-			       (begin
-				 (fresh-line)
-				 (write-string "Binary file ")
-				 (write (source-node/filename node*))
-				 (write-string " depends on ")
-				 (write (source-node/filename node))
-				 (newline)))
+			       (write-notification-line
+				(lambda (port)
+				  (write-string "Binary file " port)
+				  (write (source-node/filename node*) port)
+				  (write-string " depends on " port)
+				  (write (source-node/filename node) port))))
 			   (set-source-node/modification-time! node* #f))
 			 (source-node/forward-closure node))))
 	 source-nodes)))
@@ -248,10 +246,9 @@ USA.
 		  (pathname-delete!
 		   (pathname-new-type (source-node/pathname node) "ext"))))
 	    source-nodes/by-rank)
-  (fresh-line)
-  (newline)
-  (write-string "Begin pass 1:")
-  (newline)
+  (write-notification-line
+   (lambda (port)
+     (write-string "Begin pass 1:" port)))
   (for-each (lambda (node)
 	      (if (not (source-node/modification-time node))
 		  (source-node/syntax! node)))
@@ -261,10 +258,9 @@ USA.
 	  (and (not (source-node/modification-time node))
 	       (source-node/circular? node))))
       (begin
-	(fresh-line)
-	(newline)
-	(write-string "Begin pass 2:")
-	(newline)
+	(write-notification-line
+	 (lambda (port)
+	   (write-string "Begin pass 2:" port)))
 	(for-each (lambda (node)
 		    (if (not (source-node/modification-time node))
 			(if (source-node/circular? node)
@@ -273,31 +269,29 @@ USA.
 		  source-nodes/by-rank))))
 
 (define (source-node/touch! node)
-  (with-values
-      (lambda ()
-	(sf/pathname-defaulting (source-node/pathname node) "" #f))
-    (lambda (input-pathname bin-pathname spec-pathname)
-      input-pathname
-      (pathname-touch! bin-pathname)
-      (pathname-touch! (pathname-new-type bin-pathname "ext"))
-      (if spec-pathname (pathname-touch! spec-pathname)))))
+  (receive (input-pathname bin-pathname spec-pathname)
+      (sf/pathname-defaulting (source-node/pathname node) "" #f)
+    input-pathname
+    (pathname-touch! bin-pathname)
+    (pathname-touch! (pathname-new-type bin-pathname "ext"))
+    (if spec-pathname (pathname-touch! spec-pathname))))
 
 (define (pathname-touch! pathname)
   (if (file-exists? pathname)
       (begin
-	(fresh-line)
-	(write-string "Touch file: ")
-	(write (enough-namestring pathname))
-	(newline)
+	(write-notification-line
+	 (lambda (port)
+	   (write-string "Touch file: " port)
+	   (write (enough-namestring pathname) port)))
 	(file-touch pathname))))
 
 (define (pathname-delete! pathname)
   (if (file-exists? pathname)
       (begin
-	(fresh-line)
-	(write-string "Delete file: ")
-	(write (enough-namestring pathname))
-	(newline)
+	(write-notification-line
+	 (lambda (port)
+	   (write-string "Delete file: " port)
+	   (write (enough-namestring pathname) port)))
 	(delete-file pathname))))
 
 (define (sc filename)
@@ -305,21 +299,19 @@ USA.
   (source-node/syntax! (filename->source-node filename)))
 
 (define (source-node/syntax! node)
-  (with-values
-      (lambda ()
-	(sf/pathname-defaulting (source-node/pathname node) "" #f))
-    (lambda (input-pathname bin-pathname spec-pathname)
-      (sf/internal
-       input-pathname bin-pathname spec-pathname
-       (source-node/syntax-table node)
-       ((if compiler:enable-integration-declarations?
-	    identity-procedure
-	    (lambda (declarations)
-	      (list-transform-negative declarations
-		integration-declaration?)))
-	(source-node/declarations node))))))
+  (receive (input-pathname bin-pathname spec-pathname)
+      (sf/pathname-defaulting (source-node/pathname node) "" #f)
+    (sf/internal
+     input-pathname bin-pathname spec-pathname
+     (source-node/syntax-table node)
+     ((if compiler:enable-integration-declarations?
+	  identity-procedure
+	  (lambda (declarations)
+	    (list-transform-negative declarations
+	      integration-declaration?)))
+      (source-node/declarations node)))))
 
-(define-integrable (modification-time node type)
+(define (modification-time node type)
   (file-modification-time
    (pathname-new-type (source-node/pathname node) type)))
 
@@ -419,7 +411,7 @@ USA.
 			    "rulfix" "rulflo"
 			    "cout" "traditional" "stackify" "stackops"
 			    ))))
-    
+
     (define (file-dependency/integration/join filenames dependencies)
       (for-each (lambda (filename)
 		  (file-dependency/integration/make filename dependencies))
@@ -440,7 +432,6 @@ USA.
 
     (define-integration-dependencies "base" "object" "base" "enumer")
     (define-integration-dependencies "base" "enumer" "base" "object")
-    (define-integration-dependencies "base" "utils" "base" "scode")
     (define-integration-dependencies "base" "cfg1" "base" "object")
     (define-integration-dependencies "base" "cfg2" "base"
       "cfg1" "cfg3" "object")
@@ -452,7 +443,7 @@ USA.
     (define-integration-dependencies "base" "lvalue" "base"
       "blocks" "object" "proced" "rvalue" "utils")
     (define-integration-dependencies "base" "blocks" "base"
-      "enumer" "lvalue" "object" "proced" "rvalue" "scode")
+      "enumer" "lvalue" "object" "proced" "rvalue")
     (define-integration-dependencies "base" "proced" "base"
       "blocks" "cfg1" "cfg2" "cfg3" "contin" "enumer" "lvalue" "object"
       "rvalue" "utils")
@@ -486,7 +477,6 @@ USA.
     (define-integration-dependencies "rtlbase" "rtlreg" "rtlbase"
       "rgraph" "rtlty1")
     (define-integration-dependencies "rtlbase" "rtlty1" "rtlbase" "rtlcfg")
-    (define-integration-dependencies "rtlbase" "rtlty2" "base" "scode")
     (define-integration-dependencies "rtlbase" "rtlty2" "machines/C"
       "machin")
     (define-integration-dependencies "rtlbase" "rtlty2" "rtlbase" "rtlty1")
@@ -581,5 +571,5 @@ USA.
 	       (merge-pathnames pathname default)))
 	   integration-dependencies)))
 
-(define-integrable (integration-declaration? declaration)
+(define (integration-declaration? declaration)
   (eq? (car declaration) 'INTEGRATE-EXTERNAL))
\ No newline at end of file
