@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: buffrm.scm,v 1.66 2008/01/30 20:01:58 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -232,6 +231,9 @@ USA.
 (define-integrable (window-char->image frame char)
   (%window-char->image (frame-text-inferior frame) char))
 
+(define-integrable (window-direct-output-cursor! frame)
+  (buffer-window/direct-output-cursor! (frame-text-inferior frame)))
+
 (define-integrable (window-direct-output-forward-char! frame)
   (buffer-window/direct-output-forward-char! (frame-text-inferior frame)))
 
