@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: sendmail.scm,v 1.92 2007/09/09 16:36:50 riastradh Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -102,13 +101,31 @@ Otherwise, mail is sent directly to the named host using SMTP."
   "Service to connect to on the mail relay host.
 Can be a service name (a string), a service number, or #F.
 If #F, service \"smtp\" is used.
-This is used only of `mail-relay-host' is set."
+This is used only if `mail-relay-host' is set."
   #f
   (lambda (service)
     (or (not service)
         (exact-positive-integer? service)
         (string? service))))
 
+(define-variable mail-authentication
+  "SMTP authentication method (SASL mechanism) as a string.
+Set this to #F to disable authentication and to #T to use whatever
+  method is accepted by server, or no authentication if the server
+  accepts none.
+Currently only \"LOGIN\" and \"PLAIN\" are supported, so use this form
+  of authentication only over a secure channel."
+  #f
+  (lambda (method)
+    (member method '(#f #t "LOGIN" "PLAIN"))))
+
+(define-variable smtp-user-name
+  "User name to use for simple SMTP authentication.
+#F means prompt for the name each time."
+  #f
+  (lambda (object)
+    (or (not object) (string? object))))
+
 (define-variable smtp-trace
   "If true, direct SMTP transmissions are traced in a buffer."
   #f
@@ -120,6 +137,15 @@ Otherwise, only one valid recipient is required."
   #t
   boolean?)
 
+(define-variable smtp-greeting-hostname
+  "Hostname for HELO (or EHLO) messages when sending mail by SMTP.
+This may be a string or a procedure.
+Default is whatever the OS reports."
+  os/hostname
+  (lambda (object)
+    (or (string? object)
+	(procedure? object))))
+
 (define-variable sendmail-program
   "Filename of sendmail program."
   #f
@@ -151,6 +177,12 @@ The headers are delimited by a string found in mail-header-separator."
 		"^return-path:")
   string?)
 
+(define-variable mail-yank-prefix
+  "Prefix to insert on lines of yanked message being replied to.
+#F means use indentation."
+  #f
+  string-or-false?)
+
 (define-variable mail-interactive
   "True means when sending a message wait for and display errors.
 #F means let mailer mail back a message to report errors."
@@ -186,6 +218,11 @@ Specifically, Content-Type and Content-Transfer-Encoding headers
 If false, sent mail contains full MIME headers."
   #t
   boolean?)
+
+(define-variable-per-buffer mail-charset
+  "Name of charset for sending mail, as a string."
+  "us-ascii"
+  string?)
 
 (define-command mail
   "Edit a message to be sent.  Argument means resume editing (don't erase).
@@ -225,6 +262,13 @@ is inserted."
 
 (define (make-mail-buffer headers reply-buffer #!optional
 			  selector handle-previous buffer-name mode)
+  (make-initialized-mail-buffer headers reply-buffer
+				(lambda (buffer) buffer unspecific)
+				selector handle-previous buffer-name mode))
+
+(define (make-initialized-mail-buffer headers reply-buffer initializer
+				      #!optional selector handle-previous
+				      buffer-name mode)
   (let ((selector (if (default-object? selector) #f selector))
 	(handle-previous
 	 (if (default-object? handle-previous)
@@ -244,6 +288,7 @@ is inserted."
 					      (default-homedir-pathname))
 	       (setup-buffer-auto-save! buffer)
 	       (mail-setup buffer headers reply-buffer mode)
+	       (initializer buffer)
 	       (if (and select? selector) (selector buffer))
 	       buffer))))
       (cond ((not buffer)
@@ -273,12 +318,7 @@ is inserted."
 			      (ref-mode-object mail)))
   (local-set-variable! mail-reply-buffer reply-buffer buffer)
   (let ((headers (add-standard-headers headers buffer))
-	(point (mark-left-inserting-copy (buffer-start buffer)))
-	(fill
-	 (lambda (start end)
-	   (fill-region-as-paragraph start end
-				     "\t" (ref-variable fill-column buffer)
-				     #f))))
+	(point (mark-left-inserting-copy (buffer-start buffer))))
     (let ((start (mark-right-inserting-copy point)))
       (for-each
        (lambda (header)
@@ -310,7 +350,7 @@ is inserted."
 			      (or (string-ci=? key "to")
 				  (string-ci=? key "cc"))
 			      (caddr header)))
-		     (fill start point))
+		     (fill-mail-addresses start point))
 		 (insert-newline point)))))
        headers)
       (mark-temporary! start))
@@ -341,6 +381,27 @@ is inserted."
 	  (buffer-not-modified! buffer))))
   (event-distributor/invoke! (ref-variable mail-setup-hook buffer) buffer))
 
+(define (fill-mail-addresses start end)
+  ;; This totally loses on quoted or commented names, which it
+  ;; probably shouldn't split up.
+  (let ((column (ref-variable fill-column start))
+	(mark (char-search-forward #\, start end)))
+    (if mark
+	(let loop ((start start) (mark mark))
+	  (let ((mark* (char-search-forward #\, mark end)))
+	    (if mark*
+		(if (< (mark-column mark*) column)
+		    (loop start mark*)
+		    (let ((mark
+			   (mark-permanent-copy
+			    ;; Skip addresses that are too long.
+			    (if (mark= mark start) mark* mark))))
+		      (delete-horizontal-space mark)
+		      (insert-newline mark)
+		      (insert-char #\tab mark)
+		      (mark-temporary! mark)
+		      (loop mark mark)))))))))
+
 (define (add-standard-headers headers buffer)
   (let ((add
 	 (lambda (key value)
@@ -563,20 +624,32 @@ Here are commands that move to a header field (and create it if there isn't):
   (insert-string value (mail-new-field! header-start header-end field)))
 
 (define-command mail-yank-original
-  "Insert the message being replied to, if any (in rmail).
+  "Insert the message being replied to, if any (in rmail or imail).
 Puts point after the text and mark before.
-Indents each nonblank line ARG spaces (default 3).
-Just \\[universal-argument] as argument means don't indent
+Normally, indents each nonblank line ARG spaces (default 3).
+However, if `mail-yank-prefix' is a string, insert that prefix on each line.
+Just \\[universal-argument] as argument means don't indent, insert no prefix,
 and don't delete any header fields."
   "P"
   (lambda (argument)
-    (let ((mail-reply-buffer (ref-variable mail-reply-buffer))
-	  (left-margin
-	   (if (command-argument-multiplier-only? argument)
-	       0
-	       (or (command-argument-value argument) 3))))
+    (let ((mail-reply-buffer (ref-variable mail-reply-buffer)))
       (if mail-reply-buffer
-	  (begin
+	  (receive (prefix left-margin)
+	      (cond ((command-argument-multiplier-only? argument)
+		     (values #f 0))
+		    ((command-argument-value argument)
+		     => (lambda (v)
+			  (values #f (max 0 v))))
+		    ((ref-variable mail-yank-prefix)
+		     => (lambda (prefix)
+			  (values prefix
+				  (string-columns
+				   prefix
+				   0
+				   (ref-variable tab-width)
+				   default-char-image-strings))))
+		    (else
+		     (values #f 3)))
 	    (for-each (lambda (window)
 			(if (not (window-has-no-neighbors? window))
 			    (window-delete! window)))
@@ -597,12 +670,16 @@ and don't delete any header fields."
 		(if (not (command-argument-multiplier-only? argument))
 		    (begin
 		      (mail-yank-clear-headers start end)
-		      (indent-rigidly start end left-margin)))
+		      (if prefix
+			  (for-each-line-in-region start end
+			    (lambda (mark)
+			      (insert-string prefix mark)))
+			  (indent-rigidly start end left-margin))))
 		(mark-temporary! start)
 		(mark-temporary! end)
 		(push-current-mark! start)
 		(set-current-point! end))))))))
-
+
 (define (mail-yank-clear-headers start end)
   (let ((start (mark-left-inserting-copy start))
 	(end
@@ -637,7 +714,7 @@ Numeric argument means justify as well."
 				  (ref-variable fill-column)
 				  justify?
 				  #t))))
-
+
 (define-command mail-send-and-exit
   "Send message like mail-send, then, if no errors, exit from mail buffer.
 Prefix arg means don't delete this window."
@@ -918,46 +995,24 @@ the user from the mailer."
 
 (define (send-mail-using-smtp message-pathname recipients lookup-context)
   (message "Sending...")
-  (let ((from
-	 (rfc822:canonicalize-address-string
-	  (mail-from-string lookup-context)))
-	(trace-buffer
+  (if (null? recipients)
+      (editor-error "No recipients specified for mail."))
+  (let ((trace-buffer
 	 (and (ref-variable smtp-trace lookup-context)
-	      (temporary-buffer "*SMTP-trace*")))
-	(require-valid?
-	 (ref-variable smtp-require-valid-recipients lookup-context))
-	(valid-response?
-	 (lambda (response) (= 250 (smtp-response-number response)))))
-    (if (null? recipients)
-	(editor-error "No recipients specified for mail."))
+	      (temporary-buffer "*SMTP-trace*"))))
     (let ((responses
-	   (call-with-smtp-socket (ref-variable mail-relay-host lookup-context)
-                                  (ref-variable mail-relay-service
-                                                lookup-context)
-				  trace-buffer
-	     (lambda (port banner)
-	       banner
-	       (smtp-command/helo port)
-	       (smtp-command/mail port from)
-	       (let ((responses
-		      (map (lambda (recipient)
-			     (smtp-command/rcpt port recipient))
-			   recipients)))
-		 (if (if require-valid?
-			 (for-all? responses valid-response?)
-			 (there-exists? responses valid-response?))
-		     (smtp-command/data port message-pathname)
-		     (smtp-command/rset port))
-		 (smtp-command/quit port)
-		 responses)))))
-      (cond ((not (for-all? responses valid-response?))
+	   (transact-smtp recipients
+                          message-pathname
+                          trace-buffer
+                          lookup-context)))
+      (cond ((not (for-all? responses smtp-response-valid?))
 	     (pop-up-temporary-buffer "*SMTP-invalid*"
 				      '(READ-ONLY FLUSH-ON-SPACE)
 	       (lambda (buffer window)
 		 window
 		 (let ((m (mark-left-inserting-copy (buffer-start buffer))))
 		   (for-each (lambda (recipient response)
-			       (if (not (valid-response? response))
+			       (if (not (smtp-response-valid? response))
 				   (begin
 				     (insert-string recipient m)
 				     (insert-char #\tab m)
@@ -970,11 +1025,38 @@ the user from the mailer."
 	     (buffer-not-modified! trace-buffer)
 	     (pop-up-buffer trace-buffer #f)))
       (message "Sending..."
-	       (if (if require-valid?
-		       (for-all? responses valid-response?)
-		       (there-exists? responses valid-response?))
+	       (if (smtp-responses-ok? responses lookup-context)
 		   "done"
 		   "aborted")))))
+
+(define (transact-smtp recipients message-pathname trace-buffer lookup-context)
+  (call-with-smtp-socket (ref-variable mail-relay-host lookup-context)
+			 (ref-variable mail-relay-service lookup-context)
+			 trace-buffer
+    (lambda (port banner)
+      banner				;ignore
+      (let ((capabilities
+	     (smtp-command/ehlo port (smtp-greeting-hostname lookup-context))))
+	(smtp-authenticate port capabilities lookup-context)
+	(smtp-command/mail port (rfc822:canonicalize-address-string
+				 (mail-from-string lookup-context)))
+	(let ((responses
+	       (map (lambda (recipient)
+		      (smtp-command/rcpt port recipient))
+		    recipients)))
+	  (if (smtp-responses-ok? responses lookup-context)
+	      (smtp-command/data port message-pathname)
+	      (smtp-command/rset port))
+	  (smtp-command/quit port)
+	  responses)))))
+
+(define (smtp-response-valid? response)
+  (= 250 (smtp-response-number response)))
+
+(define (smtp-responses-ok? responses lookup-context)
+  (if (ref-variable smtp-require-valid-recipients lookup-context)
+      (for-all? responses smtp-response-valid?)
+      (there-exists? responses smtp-response-valid?)))
 
 (define (call-with-smtp-socket host-name service trace-buffer receiver)
   (let ((port #f))
@@ -982,7 +1064,7 @@ the user from the mailer."
      (lambda ()
        (set! port
 	     (make-smtp-port (open-tcp-stream-socket host-name
-                                                     (or service "smtp"))
+						     (or service "smtp"))
 			     trace-buffer))
        unspecific)
      (lambda ()
@@ -1000,6 +1082,8 @@ the user from the mailer."
 
 (define (smtp-read-line port)
   (let ((line (read-line (smtp-port-port port))))
+    (if (eof-object? line)
+	(editor-error "Premature end of input from SMTP server: " port))
     (smtp-trace-write-string line port)
     (smtp-trace-newline port)
     line))
@@ -1025,8 +1109,15 @@ the user from the mailer."
     (if trace-buffer
 	(insert-newline (buffer-end trace-buffer)))))
 
-(define (smtp-command/helo port)
-  (smtp-write-line port "HELO " (os/hostname))
+(define (smtp-command/ehlo port hostname)
+  ;++ This should probably fall back on HELO if the server answers
+  ;++ non-250, but honestly, how many non-ESMTP servers are there out
+  ;++ there?
+  (smtp-write-line port "EHLO " hostname)
+  (smtp-read-response port 250))
+
+(define (smtp-command/helo port hostname)
+  (smtp-write-line port "HELO " hostname)
   (smtp-read-response port 250))
 
 (define (smtp-command/mail port from)
@@ -1046,7 +1137,7 @@ the user from the mailer."
 	(let ((line (read-line input)))
 	  (if (not (eof-object? line))
 	      (begin
-		(if (and (fix:> 0 (string-length line))
+		(if (and (fix:> (string-length line) 0)
 			 (char=? #\. (string-ref line 0)))
 		    (smtp-write-line port "." line)
 		    (smtp-write-line port line))
@@ -1062,6 +1153,12 @@ the user from the mailer."
   (smtp-write-line port "QUIT")
   (smtp-read-response port 221))
 
+(define (smtp-greeting-hostname lookup-context)
+  (let ((hostname (ref-variable smtp-greeting-hostname lookup-context)))
+    (if (procedure? hostname)
+	(hostname)
+	hostname)))
+
 (define (smtp-read-response port . numbers)
   (smtp-drain-output port)
   (let ((response (smtp-read-line port)))
@@ -1097,7 +1194,121 @@ the user from the mailer."
 			    responses)))
 		  (cons (car lines)
 			(append-map (lambda (line) (list "\n" line))
-				    lines))))))
+				    (cdr lines)))))))
+
+;;;;; SMTP Authentication
+
+(define (smtp-authenticate port capabilities lookup-context)
+  (define (authenticate method)
+    (smtp-authenticate-with-method port method lookup-context))
+  (let ((method (ref-variable mail-authentication lookup-context)))
+    (if method
+	(let ((accepted-methods
+	       (smtp-accepted-authentication-methods capabilities)))
+	  (if (not (pair? accepted-methods))
+	      (editor-failure "No accepted authentication methods.")
+	      (cond ((and (eq? method #t)
+			  (or (member "LOGIN" accepted-methods)
+			      (member "PLAIN" accepted-methods)))
+		     => (lambda (tail)
+			  (authenticate (car tail))))
+		    ((member method accepted-methods)
+		     (authenticate method))
+		    (else
+		     (editor-failure "Authentication method not accepted:"
+				     method))))))))
+
+(define rexp:sasl-mechanism		;RFC 2222, Section 3, par. 2
+  (let ((char-set
+	 (char-set-union char-set:upper-case
+			 char-set:numeric
+			 (char-set #\- #\_))))
+    ;; The regexp compiler loses on this, unfortunately.  Fortunately,
+    ;; it doesn't matter, because if there were an excessively long
+    ;; name, it would have already caused any harm it could by the
+    ;; time we try to examine it here.
+    ;;	 (rexp-n*m 1 20 char-set)
+    (rexp+ char-set)))
+
+(define regexp:sasl-mechanism (rexp->regexp rexp:sasl-mechanism))
+
+(define rexp:smtp-auth-keywords
+  (rexp-sequence (rexp-line-start)
+		 "AUTH"
+		 (rexp-group (rexp* " " rexp:sasl-mechanism))))
+
+(define regexp:smtp-auth-keywords (rexp->regexp rexp:smtp-auth-keywords))
+
+(define (smtp-accepted-authentication-methods capabilities)
+  (let ((match
+	 (re-string-search-forward regexp:smtp-auth-keywords capabilities)))
+    (if match
+	(cdr (burst-string (re-match-extract capabilities match 1) #\space #f))
+	'())))
+
+(define (smtp-authenticate-with-method port method lookup-context)
+  (smtp-write-line port "AUTH " method)
+  (smtp-read-response port 334)
+  (let* ((user-name
+	  (or (ref-variable smtp-user-name lookup-context)
+	      (prompt-for-string "User name" #f)))
+	 (pass-phrase-key
+	  (smtp-server-pass-phrase-key user-name lookup-context)))
+    (cond ((string=? method "LOGIN")
+	   (smtp-auth:login port user-name pass-phrase-key))
+	  ((string=? method "PLAIN")
+	   (smtp-auth:plain port user-name pass-phrase-key))
+	  (else (error "Unknown SMTP authentication method:" method)))
+    (bind-condition-handler
+	(list condition-type:editor-error)
+	(lambda (condition)
+	  condition			;ignore
+	  (delete-stored-pass-phrase pass-phrase-key))
+      (lambda ()
+	(smtp-read-response port 235)))))
+
+(define (smtp-auth:login port user-name pass-phrase-key)
+  (define (base64 string)
+    (string-trim
+     (call-with-output-string
+       (lambda (port)
+	 (let ((context (encode-base64:initialize port #f)))
+	   (encode-base64:update context string 0 (string-length string))
+	   (encode-base64:finalize context))))))
+  (smtp-write-line port (base64 user-name))
+  (smtp-read-response port 334)
+  (smtp-write-line port (call-with-stored-pass-phrase pass-phrase-key base64)))
+
+(define (smtp-auth:plain port user-name pass-phrase-key)
+  ((lambda (string)
+     (smtp-write-line port (string-trim string)))
+   (call-with-output-string
+     (lambda (port)
+       (let ((context (encode-base64:initialize port)))
+	 (encode-base64:update context "\000" 0 1)
+	 (encode-base64:update context user-name 0 (string-length user-name))
+	 (encode-base64:update context "\000" 0 1)
+	 (call-with-stored-pass-phrase pass-phrase-key
+	   (lambda (pass)
+	     (encode-base64:update context pass 0 (string-length pass))))
+	 (encode-base64:finalize context))))))
+
+(define (smtp-server-pass-phrase-key user-name lookup-context)
+  ;++ Should this include `SMTP' anywhere?  Generating
+  ;++ `smtp://<user-name>@<host>:<port>' is disgusting, but it would
+  ;++ do the trick...
+  (string-append user-name
+		 "@"
+		 (ref-variable mail-relay-host lookup-context)
+		 (let ((service
+			(ref-variable mail-relay-service lookup-context)))
+		   (if (or (eqv? service 25)
+			   (equal? service "smtp"))
+		       ""
+		       (string-append ":"
+				      (if (string? service)
+					  service
+					  (number->string service #d10)))))))
 
 ;;;; MIME
 
@@ -1117,10 +1328,12 @@ the user from the mailer."
 
 (define (write-mime-message-body-1 b-start b-end subpart? port)
   (if (not (and subpart? (ref-variable mail-abbreviate-mime b-start)))
-      (write-message-header-field "Content-Type"
-				  "text/plain; charset=us-ascii"
-				  port))
-  (if (or (any-non-us-ascii-chars? b-start b-end)
+      (write-message-header-field
+       "Content-Type"
+       (string-append "text/plain; charset="
+                      (ref-variable mail-charset b-start))
+       port))
+  (if (or (any-non-printable-7bit-chars? b-start b-end)
 	  (any-lines-too-long? b-start b-end 76))
       (begin
 	(write-message-header-field "Content-Transfer-Encoding"
@@ -1146,11 +1359,11 @@ the user from the mailer."
 	(newline port)
 	(write-region-to-port b-start b-end port))))
 
-(define (any-non-us-ascii-chars? start end)
+(define (any-non-printable-7bit-chars? start end)
   (group-find-next-char-in-set (mark-group start)
 			       (mark-index start)
 			       (mark-index end)
-			       char-set:non-us-ascii))
+			       char-set:non-printable-7bit))
 
 (define (any-lines-too-long? start end n)
   (let loop ((ls (line-start start 0)))
@@ -1161,14 +1374,15 @@ the user from the mailer."
 		 (mark< ls end)
 		 (loop ls)))))))
 
-(define char-set:us-ascii
-  (char-set-union char-set:graphic (char-set #\tab #\page #\linefeed)))
+(define char-set:printable-7bit
+  (char-set-union (ascii-range->char-set #x20 #x7F)
+                  (char-set #\tab #\page #\linefeed)))
 
-(define char-set:non-us-ascii
-  (char-set-invert char-set:us-ascii))
+(define char-set:non-printable-7bit
+  (char-set-invert char-set:printable-7bit))
 
-(define regexp:non-us-ascii
-  (char-set->regexp char-set:non-us-ascii))
+(define regexp:non-printable-7bit
+  (char-set->regexp char-set:non-printable-7bit))
 
 (define (write-mime-message-body-with-attachments b-start b-end attachments
 						  port)
@@ -1336,9 +1550,9 @@ the user from the mailer."
 		 (mime-parameters->string (cdr disposition))))
 
 (define (guarantee-mime-compliant-headers h-start h-end)
-  (if (any-non-us-ascii-chars? h-start h-end)
+  (if (any-non-printable-7bit-chars? h-start h-end)
       (begin
-	(pop-up-occur-buffer h-start h-end regexp:non-us-ascii #f)
+	(pop-up-occur-buffer h-start h-end regexp:non-printable-7bit #f)
 	(editor-error "Message contains illegal characters in header.")))
   (if (any-lines-too-long? h-start h-end 998)
       (editor-error "Message contains over-long line in header.")))
@@ -1369,18 +1583,15 @@ the user from the mailer."
 	 (plen (string-length prefix)))
     (if (not (<= 1 length (- 70 plen)))
 	(error:bad-range-argument length 'RANDOM-MIME-BOUNDARY-STRING))
-    (let ((s
-	   (call-with-output-string
-	    (lambda (port)
-	      (write-string prefix port)
-	      (let ((context (encode-base64:initialize port #f)))
-		(let ((n (* (integer-ceiling (- length 2) 4) 3)))
-		  (encode-base64:update context (random-byte-vector n) 0 n))
-		(encode-base64:finalize context)))))
-	  (n (+ plen length)))
-      (if (fix:> (string-length s) n)
-	  (set-string-maximum-length! s n))
-      s)))
+    (string-head! (call-with-output-string
+		    (lambda (port)
+		      (write-string prefix port)
+		      (let ((context (encode-base64:initialize port #f)))
+			(let ((n (* (integer-ceiling (- length 2) 4) 3)))
+			  (encode-base64:update context
+						(random-byte-vector n) 0 n))
+			(encode-base64:finalize context))))
+		  (+ plen length))))
 
 ;;;; Attachment browser
 
@@ -1597,9 +1808,26 @@ Otherwise, the MIME type is determined from the file's suffix;
 			      (else (editor-beep) (loop))))))))))))
     (values mime-type
 	    (if (eq? (mime-type/top-level mime-type) 'TEXT)
-		'((CHARSET "iso-8859-1"))
+		(let ((charset
+		       (ref-variable default-mime-text-charset buffer)))
+		  (if (not charset)
+		      '()
+		      `((CHARSET
+			 ,(if (eq? charset 'PROMPT)
+			      (prompt-for-string "Charset:" "ISO-8859-1")
+			      charset)))))
 		'()))))
 
+(define-variable default-mime-text-charset
+  "Default charset for MIME text/plain entities, as a string.
+If #F, no charset is included.
+If the symbol PROMPT, prompt for a charset."
+  "ISO-8859-1"
+  (lambda (object)
+    (or (string? object)
+	(eq? object 'PROMPT)
+	(eq? object #f))))
+
 (define-variable file-type-to-mime-type
   "Specifies the MIME type/subtype for files with a given type.
 This is a list, each element of which is a list of three items:
