@@ -1,8 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: wincom.scm,v 1.135 2003/02/14 18:28:14 cph Exp $
-
-Copyright 1987, 1989-2000 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -335,6 +336,13 @@ or if the window is the only window of its frame."
   ()
   (lambda () (delete-other-windows (selected-window))))
 
+(define-command kill-buffer-and-window
+  "Kill the current buffer and delete the window it was in."
+  ()
+  (lambda ()
+    (kill-buffer-interactive (current-buffer))
+    (window-delete! (selected-window))))
+
 (define-command other-window
   "Select the ARG'th different window."
   "p"
