@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: imail-core.scm,v 1.178 2008/09/25 15:16:08 riastradh Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -407,7 +406,7 @@ USA.
   (hash-table/remove! memoized-resources url))
 
 (define memoized-resources
-  (make-eq-hash-table))
+  (make-weak-eq-hash-table))
 
 ;;;; Folder operations
 
@@ -631,11 +630,52 @@ USA.
    (lambda (port)
      (write-header-fields (message-header-fields message) port)
      (write-message-body message port))))
-
+
 (define (message-time message)
-  (let ((date (get-first-header-field-value message "date" #f)))
-    (and date
-	 (parse-header-field-date date))))
+  (intern-property! message '|#[(edwin imail)message-time]|
+    (lambda (message)
+      (let ((date (get-first-header-field-value message "date" #f)))
+	(and date
+	     (parse-header-field-date date))))))
+
+(define (message-subject message)
+  (intern-property! message '|#[(edwin imail)message-subject]|
+    (lambda (message)
+      (cond ((get-first-header-field-value message "subject" #f)
+	     => strip-subject-re)
+	    (else "")))))
+
+(define (strip-subject-re subject)
+  (let ((end (string-length subject)))
+    (let loop ((start 0))
+      (if (and (<= 3 (- end start))
+	       (substring-prefix-ci? "Re:" 0 3 subject start end))
+	  (cond ((substring-find-next-char-in-set subject (+ start 3) end
+						  char-set:subject-content)
+		 => loop)
+		(else ""))
+	  (string-tail subject start)))))
+
+(define char-set:subject-content (char-set-invert (char-set #\space #\tab)))
+
+(define (message-author message)
+  (intern-property! message '|#[(edwin imail)message-author]|
+    (lambda (message)
+      (or (get-first-header-field-address message "from" #f)
+	  (get-first-header-field-address message "sender" #f)
+	  ""))))
+
+(define (message-recipient message)
+  (intern-property! message '|#[(edwin imail)message-recipient]|
+    (lambda ()
+      (or (get-first-header-field-address message "to" #f)
+	  (get-first-header-field-address message "apparently-to" #f)
+	  ""))))
+
+(define (get-first-header-field-address message name error?)
+  (let ((v (get-first-header-field-value message name error?)))
+    (and v
+	 (rfc822:first-address v))))
 
 ;;;; Message Navigation
 
