@@ -1,10 +1,9 @@
 /* -*-C-*-
 
-$Id: ntio.c,v 1.36 2008/01/30 20:02:15 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -65,7 +64,7 @@ static int test_for_pending_event (void);
 #define NT_DEFAULT_CHANNEL_TABLE_SIZE 1024
 #endif
 
-size_t OS_channel_table_size;
+Tchannel OS_channel_table_size;
 struct channel * NT_channel_table;
 
 Tchannel
@@ -225,6 +224,11 @@ OS_channel_type (Tchannel channel)
 {
   return (CHANNEL_TYPE (channel));
 }
+
+void
+OS_channel_synchronize (Tchannel channel)
+{
+}
 
 static void
 generic_channel_close (Tchannel channel, int errorp)
