@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -666,7 +667,7 @@ pc_in_hyperspace:
 
   scheme_sp_valid
     = (pc_in_scheme
-       && (ADDRESS_IN_STACK_P (scheme_sp))
+       && (ADDRESS_IN_STACK_P ((SCHEME_OBJECT *) scheme_sp))
        && (ALIGNED_P (scheme_sp)));
 
   IFVERBOSE (TellUserEx (MB_OKCANCEL, "continue_from_trap 2"));
