@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -92,7 +93,7 @@ USA.
 	       (fix:xor (vector-8b-ref buf k)
 			(vector-ref s (inc-mod
 				       (fix:+ (fix:1+ (vector-ref s i))
-					      (vector-ref s j)) 
+					      (vector-ref s j))
 				       ts))))
 	      (loop (fix:1+ k) i j)))
 	  (begin
@@ -107,7 +108,7 @@ USA.
      (vector-ref '#("Mon" "Tue" "Wed" "Thu" "Fri" "Sat" "Sun")
 		 (decoded-time/day-of-week the-time))
      " "
-     (vector-ref '#("Jan" "Feb" "Mar" "Apr" "May" "Jun" 
+     (vector-ref '#("Jan" "Feb" "Mar" "Apr" "May" "Jun"
 			  "Jul" "Aug" "Sep" "Oct" "Nov" "Dec")
 	       (-1+ (decoded-time/month the-time)))
      " "
