@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: make.scm,v 14.112 2008/01/30 20:02:32 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -30,13 +29,13 @@ USA.
 
 (declare (usual-integrations))
 
-(set-interrupt-enables! 0)
+((ucode-primitive set-interrupt-enables!) 0)
 
 ;; This must be defined as follows so that it is no part of a multi-define
 ;; itself.  It must also precede any other top-level definitions in this file
 ;; that are not performed directly using LOCAL-ASSIGNMENT.
 
-(local-assignment
+((ucode-primitive local-assignment)
  #f ;global environment
  'DEFINE-MULTIPLE
  (lambda (env names values)
@@ -48,9 +47,9 @@ USA.
      (let loop ((i 0) (val unspecific))
        (if (fix:< i len)
 	   (loop (fix:+ i 1)
-		 (local-assignment env
-				   (vector-ref names i)
-				   (vector-ref values i)))
+		 ((ucode-primitive local-assignment) env
+						     (vector-ref names i)
+						     (vector-ref values i)))
 	   val)))))
 
 (define system-global-environment #f)
@@ -60,37 +59,20 @@ USA.
 ;; *MAKE-ENVIRONMENT is referred to by compiled code.  It must go
 ;; before the uses of the-environment later, and after apply above.
 (define (*make-environment parent names . values)
-  (let-syntax
-      ((ucode-type
-	(sc-macro-transformer
-	 (lambda (form environment)
-	   environment
-	   (microcode-type (cadr form))))))
-    (system-list->vector
-     (ucode-type environment)
-     (cons (system-pair-cons (ucode-type procedure)
-			     (system-pair-cons (ucode-type lambda)
-					       unspecific
-					       names)
-			     parent)
-	   values))))
+  ((ucode-primitive system-list-to-vector)
+   (ucode-type environment)
+   (cons ((ucode-primitive system-pair-cons)
+	  (ucode-type procedure)
+	  ((ucode-primitive system-pair-cons) (ucode-type lambda)
+					      unspecific
+					      names)
+	  parent)
+	 values)))
 
 (let ((environment-for-package
        (*make-environment system-global-environment
 			  (vector lambda-tag:unnamed))))
 
-(define-syntax ucode-primitive
-  (sc-macro-transformer
-   (lambda (form environment)
-     environment
-     (apply make-primitive-procedure (cdr form)))))
-
-(define-syntax ucode-type
-  (sc-macro-transformer
-   (lambda (form environment)
-     environment
-     (microcode-type (cadr form)))))
-
 (define-integrable + (ucode-primitive integer-add))
 (define-integrable - (ucode-primitive integer-subtract))
 (define-integrable < (ucode-primitive integer-less?))
@@ -189,36 +171,53 @@ USA.
 	    (loop (cdr name)))))
     (tty-write-string ")"))
 
-  (let ((env (package-reference package-name)))
-    (cond ((not (lexical-unreferenceable? env procedure-name))
-	   (print-name "initialize:")
-	   (if (not (eq? procedure-name 'INITIALIZE-PACKAGE!))
-	       (begin
-		 (tty-write-string " [")
-		 (tty-write-string (system-pair-car procedure-name))
-		 (tty-write-string "]")))
-	   ((lexical-reference env procedure-name)))
-	  ((not mandatory?)
-	   (print-name "* skipping:"))
-	  (else
-	   ;; Missing mandatory package! Report it and die.
-	   (print-name "Package")
-	   (tty-write-string " is missing initialization procedure ")
-	   (tty-write-string (system-pair-car procedure-name))
-	   (fatal-error "Could not initialize a required package.")))))
+  (cond ((let ((package (find-package package-name #f)))
+	   (and package
+		(let ((env (package/environment package)))
+		  (if (not procedure-name)
+		      (if (lexical-unreferenceable? env 'INITIALIZE-PACKAGE!)
+			  (lambda () ((access run-boot-inits! boot-defs) env))
+			  (lexical-reference env 'INITIALIZE-PACKAGE!))
+		      (and (not (lexical-unreferenceable? env procedure-name))
+			   (lexical-reference env procedure-name))))))
+	 => (lambda (procedure)
+	      (print-name "initialize:")
+	      (if (not (or (not procedure-name)
+			   (eq? procedure-name 'INITIALIZE-PACKAGE!)))
+		  (begin
+		    (tty-write-string " [")
+		    (tty-write-string (system-pair-car procedure-name))
+		    (tty-write-string "]")))
+	      (procedure)))
+	((not mandatory?)
+	 (print-name "* skipping:"))
+	(else
+	 ;; Missing mandatory package! Report it and die.
+	 (print-name "Package")
+	 (tty-write-string " is missing initialization procedure ")
+	 (if procedure-name
+	     (tty-write-string (system-pair-car procedure-name)))
+	 (fatal-error "Could not initialize a required package."))))
 
 (define (package-reference name)
   (package/environment (find-package name)))
 
 (define (package-initialization-sequence specs)
-  (let loop ((specs specs))
-    (if (pair? specs)
-	(let ((spec (car specs)))
-	  (if (or (not (pair? spec))
-		  (symbol? (car spec)))
-	      (package-initialize spec 'INITIALIZE-PACKAGE! #f)
-	      (package-initialize (car spec) (cadr spec) (caddr spec)))
-	  (loop (cdr specs))))))
+  (do ((specs specs (cdr specs)))
+      ((not (pair? specs)) unspecific)
+    (let ((spec (car specs)))
+      (cond ((eq? (car spec) 'OPTIONAL)
+	     (package-initialize (cadr spec)
+				 (and (pair? (cddr spec))
+				      (caddr spec))
+				 #f))
+	    ((pair? (car spec))
+	     (package-initialize (car spec)
+				 (and (pair? (cdr spec))
+				      (cadr spec))
+				 #t))
+	    (else
+	     (package-initialize spec #f #t))))))
 
 (define (remember-to-purify purify? filename value)
   (if purify?
@@ -296,7 +295,8 @@ USA.
 
 (define (implemented-primitive-procedure? primitive)
   ((ucode-primitive get-primitive-address)
-   (intern ((ucode-primitive get-primitive-name) (object-datum primitive)))
+   (intern ((ucode-primitive get-primitive-name)
+	    ((ucode-primitive object-datum) primitive)))
    #f))
 
 (define initialize-c-compiled-block
@@ -355,6 +355,7 @@ USA.
  packages-file)
 
 ;;; Global databases.  Load, then initialize.
+(define boot-defs)
 (let ((files1
        '(("gcdemn" . (RUNTIME GC-DAEMONS))
 	 ("gc" . (RUNTIME GARBAGE-COLLECTOR))
@@ -368,10 +369,11 @@ USA.
 	 ("random" . (RUNTIME RANDOM-NUMBER))
 	 ("gentag" . (RUNTIME GENERIC-PROCEDURE))
 	 ("poplat" . (RUNTIME POPULATION))
-	 ("record" . (RUNTIME RECORD))
-	 ("syntax-transforms" . (RUNTIME SYNTACTIC-CLOSURES))))
+	 ("record" . (RUNTIME RECORD))))
       (files2
-       '(("prop1d" . (RUNTIME 1D-PROPERTY))
+       '(("syntax-items" . (RUNTIME SYNTAX ITEMS))
+	 ("syntax-transforms" . (RUNTIME SYNTAX TRANSFORMS))
+	 ("prop1d" . (RUNTIME 1D-PROPERTY))
 	 ("events" . (RUNTIME EVENT-DISTRIBUTOR))
 	 ("gdatab" . (RUNTIME GLOBAL-DATABASE))
 	 ("gcfinal" . (RUNTIME GC-FINALIZER))
@@ -383,24 +385,24 @@ USA.
 	   (eval (file->object (car (car files)) #t #t)
 		 (package-reference (cdr (car files))))))))
   (load-files files1)
-  (package-initialize '(RUNTIME GC-DAEMONS) 'INITIALIZE-PACKAGE! #t)
-  (package-initialize '(RUNTIME GARBAGE-COLLECTOR) 'INITIALIZE-PACKAGE! #t)
-  (package-initialize '(RUNTIME RANDOM-NUMBER) 'INITIALIZE-PACKAGE! #t)
+  (package-initialize '(RUNTIME GC-DAEMONS) #f #t)
+  (package-initialize '(RUNTIME GARBAGE-COLLECTOR) #f #t)
+  (package-initialize '(RUNTIME RANDOM-NUMBER) #f #t)
   (package-initialize '(RUNTIME GENERIC-PROCEDURE) 'INITIALIZE-TAG-CONSTANTS!
 		      #t)
-  (package-initialize '(RUNTIME POPULATION) 'INITIALIZE-PACKAGE! #t)
+  (package-initialize '(RUNTIME POPULATION) #f #t)
   (package-initialize '(RUNTIME RECORD) 'INITIALIZE-RECORD-TYPE-TYPE! #t)
-  (package-initialize '(RUNTIME SYNTACTIC-CLOSURES)
-		      'INITIALIZE-SYNTAX-TRANSFORMS!
-		      #t)
   (load-files files2)
-  (package-initialize '(RUNTIME 1D-PROPERTY) 'INITIALIZE-PACKAGE! #t)
-  (package-initialize '(RUNTIME EVENT-DISTRIBUTOR) 'INITIALIZE-PACKAGE! #t)
-  (package-initialize '(RUNTIME GLOBAL-DATABASE) 'INITIALIZE-PACKAGE! #t)
+  (package-initialize '(RUNTIME 1D-PROPERTY) #f #t)
+  (package-initialize '(RUNTIME EVENT-DISTRIBUTOR) #f #t)
+  (package-initialize '(RUNTIME GLOBAL-DATABASE) #f #t)
   (package-initialize '(RUNTIME POPULATION) 'INITIALIZE-UNPARSER! #t)
   (package-initialize '(RUNTIME 1D-PROPERTY) 'INITIALIZE-UNPARSER! #t)
-  (package-initialize '(RUNTIME GC-FINALIZER) 'INITIALIZE-PACKAGE! #t)
-  (package-initialize '(RUNTIME STRING) 'INITIALIZE-PACKAGE! #t)
+  (package-initialize '(RUNTIME GC-FINALIZER) #f #t)
+  (package-initialize '(RUNTIME STRING) #f #t)
+
+  (set! boot-defs
+	(package/environment (name->package '(RUNTIME BOOT-DEFINITIONS))))
 
   ;; Load everything else.
   ((lexical-reference environment-for-package 'LOAD-PACKAGES-FROM-FILE)
@@ -419,24 +421,27 @@ USA.
 		    (string=? filename "packag")
 		    (file-member? filename files1)
 		    (file-member? filename files2)))
-	   (eval (file->object filename #t #t)
-		 environment))
+	   (begin
+	     ((access init-boot-inits! boot-defs))
+	     (eval (file->object filename #t #t)
+		   environment)
+	     ((access save-boot-inits! boot-defs) environment)))
        unspecific))))
 
 ;;; Funny stuff is done.  Rest of sequence is standardized.
 (package-initialization-sequence
  '(
    ;; Microcode interface
-   ((RUNTIME MICROCODE-TABLES) READ-MICROCODE-TABLES! #t)
+   ((RUNTIME MICROCODE-TABLES) READ-MICROCODE-TABLES!)
    (RUNTIME STATE-SPACE)
    (RUNTIME APPLY)
    (RUNTIME HASH)			; First GC daemon!
    (RUNTIME PRIMITIVE-IO)
    (RUNTIME SYSTEM-CLOCK)
-   ((RUNTIME GC-FINALIZER) INITIALIZE-EVENTS! #t)
+   ((RUNTIME GC-FINALIZER) INITIALIZE-EVENTS!)
    ;; Basic data structures
    (RUNTIME NUMBER)
-   ((RUNTIME NUMBER) INITIALIZE-DRAGON4! #t)
+   ((RUNTIME NUMBER) INITIALIZE-DRAGON4!)
    (RUNTIME MISCELLANEOUS-GLOBAL)
    (RUNTIME CHARACTER)
    (RUNTIME CHARACTER-SET)
@@ -444,6 +449,7 @@ USA.
    (RUNTIME STREAM)
    (RUNTIME 2D-PROPERTY)
    (RUNTIME HASH-TABLE)
+   (RUNTIME REGULAR-SEXPRESSION)
    ;; Microcode data structures
    (RUNTIME HISTORY)
    (RUNTIME LAMBDA-ABSTRACTION)
@@ -453,52 +459,57 @@ USA.
    (RUNTIME CONTINUATION-PARSER)
    (RUNTIME PROGRAM-COPIER)
    ;; Generic Procedures
-   ((RUNTIME GENERIC-PROCEDURE EQHT) INITIALIZE-ADDRESS-HASHING! #t)
-   ((RUNTIME GENERIC-PROCEDURE) INITIALIZE-GENERIC-PROCEDURES! #t)
-   ((RUNTIME GENERIC-PROCEDURE MULTIPLEXER) INITIALIZE-MULTIPLEXER! #t)
-   ((RUNTIME TAGGED-VECTOR) INITIALIZE-TAGGED-VECTOR! #t)
-   ((RUNTIME RECORD-SLOT-ACCESS) INITIALIZE-RECORD-SLOT-ACCESS! #t)
-   ((RUNTIME RECORD) INITIALIZE-RECORD-PROCEDURES! #t)
-   ((PACKAGE) FINALIZE-PACKAGE-RECORD-TYPE! #t)
-   ((RUNTIME RANDOM-NUMBER) FINALIZE-RANDOM-STATE-TYPE! #t)
+   ((RUNTIME GENERIC-PROCEDURE EQHT) INITIALIZE-ADDRESS-HASHING!)
+   ((RUNTIME GENERIC-PROCEDURE) INITIALIZE-GENERIC-PROCEDURES!)
+   ((RUNTIME GENERIC-PROCEDURE MULTIPLEXER) INITIALIZE-MULTIPLEXER!)
+   ((RUNTIME TAGGED-VECTOR) INITIALIZE-TAGGED-VECTOR!)
+   ((RUNTIME RECORD-SLOT-ACCESS) INITIALIZE-RECORD-SLOT-ACCESS!)
+   ((RUNTIME RECORD) INITIALIZE-RECORD-PROCEDURES!)
+   ((PACKAGE) FINALIZE-PACKAGE-RECORD-TYPE!)
+   ((RUNTIME RANDOM-NUMBER) FINALIZE-RANDOM-STATE-TYPE!)
    ;; Condition System
    (RUNTIME ERROR-HANDLER)
    (RUNTIME MICROCODE-ERRORS)
-   ((RUNTIME GENERIC-PROCEDURE) INITIALIZE-CONDITIONS! #t)
-   ((RUNTIME GENERIC-PROCEDURE MULTIPLEXER) INITIALIZE-CONDITIONS! #t)
-   ((RUNTIME RECORD-SLOT-ACCESS) INITIALIZE-CONDITIONS! #t)
-   ((RUNTIME STREAM) INITIALIZE-CONDITIONS! #t)
+   ((RUNTIME GENERIC-PROCEDURE) INITIALIZE-CONDITIONS!)
+   ((RUNTIME GENERIC-PROCEDURE MULTIPLEXER) INITIALIZE-CONDITIONS!)
+   ((RUNTIME RECORD-SLOT-ACCESS) INITIALIZE-CONDITIONS!)
+   ((RUNTIME STREAM) INITIALIZE-CONDITIONS!)
+   ((RUNTIME REGULAR-SEXPRESSION) INITIALIZE-CONDITIONS!)
    ;; System dependent stuff
-   ((RUNTIME OS-PRIMITIVES) INITIALIZE-SYSTEM-PRIMITIVES! #t)
+   ((RUNTIME OS-PRIMITIVES) INITIALIZE-SYSTEM-PRIMITIVES!)
+   ;; Floating-point environment -- needed by threads.
+   (RUNTIME FLOATING-POINT-ENVIRONMENT)
    ;; Threads
    (RUNTIME THREAD)
    ;; I/O
+   (RUNTIME OUTPUT-PORT)
    (RUNTIME GENERIC-I/O-PORT)
    (RUNTIME FILE-I/O-PORT)
    (RUNTIME CONSOLE-I/O-PORT)
    (RUNTIME SOCKET)
    (RUNTIME TRANSCRIPT)
-   (RUNTIME STRING-INPUT)
-   (RUNTIME STRING-OUTPUT)
-   (RUNTIME TRUNCATED-STRING-OUTPUT)
-   ;; These MUST be done before (RUNTIME PATHNAME) 
+   (RUNTIME STRING-I/O-PORT)
+   (RUNTIME USER-INTERFACE)
+   ;; These MUST be done before (RUNTIME PATHNAME)
    ;; Typically only one of them is loaded.
    (RUNTIME PATHNAME UNIX)
    (RUNTIME PATHNAME DOS)
    (RUNTIME PATHNAME)
    (RUNTIME WORKING-DIRECTORY)
    (RUNTIME LOAD)
-   (RUNTIME UNICODE)
    (RUNTIME SIMPLE-FILE-OPS)
-   ((RUNTIME OS-PRIMITIVES) INITIALIZE-MIME-TYPES! #f)
+   (OPTIONAL (RUNTIME OS-PRIMITIVES) INITIALIZE-MIME-TYPES!)
    ;; Syntax
+   (RUNTIME KEYWORD)
    (RUNTIME NUMBER-PARSER)
    (RUNTIME PARSER)
-   ((RUNTIME PATHNAME) INITIALIZE-PARSER-METHOD! #t)
+   (RUNTIME PARSER FILE-ATTRIBUTES)
+   ((RUNTIME PATHNAME) INITIALIZE-PARSER-METHOD!)
    (RUNTIME UNPARSER)
    (RUNTIME UNSYNTAXER)
    (RUNTIME PRETTY-PRINTER)
    (RUNTIME EXTENDED-SCODE-EVAL)
+   (RUNTIME SYNTAX DEFINITIONS)
    ;; REP Loops
    (RUNTIME INTERRUPT-HANDLER)
    (RUNTIME GC-STATISTICS)
@@ -517,18 +528,22 @@ USA.
    ;; Graphics.  The last type initialized is the default for
    ;; MAKE-GRAPHICS-DEVICE, only the types that are valid for the
    ;; operating system are actually loaded and initialized.
-   (RUNTIME STARBASE-GRAPHICS)
-   (RUNTIME X-GRAPHICS)
-   (RUNTIME OS2-GRAPHICS)
+   (OPTIONAL (RUNTIME STARBASE-GRAPHICS))
+   (OPTIONAL (RUNTIME X-GRAPHICS))
+   (OPTIONAL (RUNTIME OS2-GRAPHICS))
    ;; Emacs -- last because it installs hooks everywhere which must be initted.
    (RUNTIME EMACS-INTERFACE)
    ;; More debugging
-   ((RUNTIME CONTINUATION-PARSER) INITIALIZE-SPECIAL-FRAMES! #f)
+   (OPTIONAL (RUNTIME CONTINUATION-PARSER) INITIALIZE-SPECIAL-FRAMES!)
    (RUNTIME URI)
-   (RUNTIME HTTP-CLIENT)))
-
-(if (eq? os-name 'NT)
-    (package-initialize '(RUNTIME WIN32-REGISTRY) 'INITIALIZE-PACKAGE! #f))
+   (RUNTIME RFC2822-HEADERS)
+   (RUNTIME HTTP-SYNTAX)
+   (RUNTIME HTTP-CLIENT)
+   (RUNTIME HTML-FORM-CODEC)
+   (OPTIONAL (RUNTIME WIN32-REGISTRY))
+   (OPTIONAL (RUNTIME FFI))
+   (RUNTIME SWANK)
+   (RUNTIME STACK-SAMPLER)))
 
 (let ((obj (file->object "site" #t #f)))
   (if obj
@@ -570,5 +585,8 @@ USA.
 )
 
 (package/add-child! system-global-package 'USER user-initial-environment)
+;; Might be better to do this sooner, to trap on floating-point
+;; mistakes earlier in the cold load.
+(flo:set-environment! (flo:default-environment))
 (start-thread-timer)
 (initial-top-level-repl)
\ No newline at end of file
