@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -33,10 +34,10 @@ USA.
 (add-boot-init!
  (lambda ()
    (set! copyright-years
-	 (let ((now 2010)
+	 (let ((now 2011)
 	       (then 1986))
 	   (iota (+ (- now then) 1) then)))
-   (add-subsystem-identification! "Release" '(9 0))
+   (add-subsystem-identification! "Release" '(9 1))
    (snarf-microcode-version!)
    (add-event-receiver! event:after-restore snarf-microcode-version!)
    (add-subsystem-identification! "Runtime" '(15 7))))
@@ -174,7 +175,7 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.")
 (define (write-words words line-prefix indentation port)
   (let ((wrap-column (- (output-port/x-size port) 5))
 	(space " "))
-    
+
     (define (write-first-word words indent?)
       (write-string line-prefix port)
       (if indent? (write-string indentation port))
